@@ -8,7 +8,7 @@ package Inline::Java ;
 use strict ;
 require 5.006 ;
 
-$Inline::Java::VERSION = '0.50' ;
+$Inline::Java::VERSION = '0.51' ;
 
 
 # DEBUG is set via the DEBUG config
@@ -31,6 +31,7 @@ use Inline::Java::Portable ;
 use Inline::Java::Class ;
 use Inline::Java::Object ;
 use Inline::Java::Array ;
+use Inline::Java::Handle ;
 use Inline::Java::Protocol ;
 use Inline::Java::Callback ;
 # Must be last.
@@ -136,6 +137,7 @@ sub validate {
 	$o->set_option('J2SDK',					$jdk,			's', 1, \%opts) ;
 	$o->set_option('CLASSPATH',				'',				's', 1, \%opts) ;
 
+	$o->set_option('BIND',					'localhost',	's', 1, \%opts) ;
 	$o->set_option('HOST',					'localhost',	's', 1, \%opts) ;
 	$o->set_option('PORT',					-1,				'i', 1, \%opts) ;
 	$o->set_option('STARTUP_DELAY',			15,				'i', 1, \%opts) ;
@@ -143,6 +145,7 @@ sub validate {
 	$o->set_option('START_JVM',				1,				'b', 1, \%opts) ;
 	$o->set_option('JNI',					0,				'b', 1, \%opts) ;
 	$o->set_option('EMBEDDED_JNI',			0,				'b', 1, \%opts) ;
+	$o->set_option('NATIVE_DOUBLES',		0,				'b', 1, \%opts) ;
 
 	$o->set_option('WARN_METHOD_SELECT',	0,				'b', 1, \%opts) ;
 	$o->set_option('STUDY',					undef,			'a', 0, \%opts) ;
@@ -341,8 +344,8 @@ sub build {
 	$pcode =~ s/\"(.*?)\"//g ;
 	$pcode =~ s/\/\*(.*?)\*\///gs ;
 	$pcode =~ s/\/\/(.*)$//gm ;
-	if ($pcode =~ /public\s+class\s+(\w+)/){
-		$source = "$1.java" ;
+	if ($pcode =~ /public\s+(abstract\s+)?class\s+(\w+)/){
+		$source = "$2.java" ;
 	}
 
 	my $install_dir = File::Spec->catdir($o->get_api('install_lib'), 
@@ -357,8 +360,9 @@ sub build {
 		close(Inline::Java::JAVA) ;
 
 		# ... and compile it.
-		my $javac = File::Spec->catfile($o->get_java_config('J2SDK'), 'bin', 
-		"javac" . Inline::Java::Portable::portable("EXE_EXTENSION")) ;
+		my $javac = File::Spec->catfile($o->get_java_config('J2SDK'), 
+			Inline::Java::Portable::portable("J2SDK_BIN"), 
+			"javac" . Inline::Java::Portable::portable("EXE_EXTENSION")) ;
 		my $redir = Inline::Java::Portable::portable("IO_REDIR") ;
 
 		my $args = "-deprecation " . $o->get_java_config('EXTRA_JAVAC_ARGS') ;
