@@ -6,9 +6,10 @@ use Carp ;
 use IO::File ;
 use IPC::Open3 ;
 use IO::Socket ;
+use Text::ParseWords ;
 use Inline::Java::Portable ;
 
-$Inline::Java::JVM::VERSION = '0.50' ;
+$Inline::Java::JVM::VERSION = '0.51' ;
 
 my %SIGS = () ;
 
@@ -47,17 +48,21 @@ sub new {
 	if ($o->get_java_config('JNI')){
 		Inline::Java::debug(1, "JNI mode") ;
 
+		# Split args and remove quotes
+		my @args = map {s/(['"])(.*)\1/$2/ ; $_}
+			parse_line('\s+', 1, $args) ;
 		my $jni = new Inline::Java::JNI(
 			$ENV{CLASSPATH} || '',
-			$args || '',
+			\@args,
 			$this->{embedded},
 			Inline::Java::get_DEBUG(),
+			$o->get_java_config('NATIVE_DOUBLES'),
 		) ;
 		$jni->create_ijs() ;
 
 		$this->{JNI} = $jni ;
 	}
-	else{
+	else {
 		Inline::Java::debug(1, "client/server mode") ;
 
 		my $debug = Inline::Java::get_DEBUG() ;
@@ -67,6 +72,9 @@ sub new {
 		$this->{port} = $o->get_java_config('PORT') ;
 		$this->{host} = $o->get_java_config('HOST') ;
 
+		# Used to limit the bind of the JVM server
+		$this->{'bind'} = $o->get_java_config('BIND') ;
+
 		# Grab the next free port number and release it.
 		if ((! $this->{shared})&&($this->{port} < 0)){
 			if (Inline::Java::Portable::portable("GOT_NEXT_FREE_PORT")){
@@ -109,13 +117,15 @@ sub new {
 			}
 		}
 
-		my $java = File::Spec->catfile($o->get_java_config('J2SDK'), 'bin',
+		my $java = File::Spec->catfile($o->get_java_config('J2SDK'), 
+			Inline::Java::Portable::portable("J2SDK_BIN"),
 			($this->{debugger} ? "jdb" : "java") . 
 			Inline::Java::Portable::portable("EXE_EXTENSION")) ;
 
 		my $shared = ($this->{shared} ? "true" : "false") ;
 		my $priv = ($this->{private} ? "true" : "false") ;
-		my $cmd = Inline::Java::Portable::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{port} $shared $priv") ;
+		my $native_doubles = ($o->get_java_config('NATIVE_DOUBLES') ? "true" : "false") ;
+		my $cmd = Inline::Java::Portable::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{bind} $this->{port} $shared $priv $native_doubles") ;
 		Inline::Java::debug(2, $cmd) ;
 		if ($o->get_config('UNTAINT')){
 			($cmd) = $cmd =~ /(.*)/ ;
@@ -421,7 +431,7 @@ sub process_command {
 		Inline::Java::debug(3, "packet recv is $resp") ;
 
 		# We got an answer from the server. Is it a callback?
-		if ($resp =~ /^callback/){
+		if ($resp =~ /^callback/o){
 			($data, $Inline::Java::Callback::OBJECT_HOOK) = Inline::Java::Callback::InterceptCallback($inline, $resp) ;
 			next ;
 		}
